#!/bin/bash

# This script generates the absfrontend.jar from the hats repository 
# and copies it to the costa/costabs/lib directory
#author Antonio Flores
#license GPL

#
#use:
#./libUpdate [-c costa_path] [-h hats_path]
# where costa_path and hats_path are the absolute paths to the root of the respective systems

COSTA_DIR="$HOME/Systems/costa"

HATS_DIR="$HOME/Projects/EU/HATS/HATS-global"


while getopts "c:h:" optname
  do
    case "$optname" in
      "c")
        COSTA_DIR=$OPTARG
        ;;
      "h")
        HATS_DIR=$OPTARG
        ;;
      "?")
	echo "Use:"
	echo "libUpdate [-c costa_path] [-h hats_path]"
	echo "In case an option is not specified the default values are set to:"
	echo 'costa_path=$HOME/Systems/costa\n'
	echo 'hats_path=$HOME/Projects/EU/HATS/HATS-global\n'
	exit
        ;;
      ":")
	echo "Use\n libUpdate [-c costa_path] [-h hats_path]"
	echo "libUpdate [-c costa_path] [-h hats_path]"
	echo "In case an option is not specified the default values are set"
	echo 'costa_path=$HOME/Systems/costa'
	echo 'hats_path=$HOME/Projects/EU/HATS/HATS-global'
	exit
        ;;
      "*")
      # Should not occur
        echo "Unknown error while processing options" 1>&2
	exit
        ;;
    esac
  done

COSTA_ABS_LIB_DIR="$COSTA_DIR/costabs/lib/"
HATS_DIR_FRONTEND="$HATS_DIR/Tools/ABS/trunk/frontend/"

if [ -d $COSTA_ABS_LIB_DIR ]
	then
	if [ -d $HATS_DIR_FRONTEND ]
	then
		cd $HATS_DIR_FRONTEND 
		ant clean
		ant
		ant dist
		cd "$COSTA_ABS_LIB_DIR"
		cp "$HATS_DIR_FRONTEND/dist/absfrontend.jar"  .
	else
	echo "Error: The hats directory $HATS_DIR_FRONTEND does not exist" 1>&2

	fi
else
echo "Error: The costa directory $COSTA_ABS_LIB_DIR does not exist" 1>&2
fi
exit
