/** <module> The Costabs parameters

This module provides support for handling the parameters of
Costabs. Costabs first sets them to their default values (predicate
set_default_costabs_params/0), then the driver optionally sets them
from the input list of parameters. The current value of a parameter is
looked up at any point by means of predicate costabs_param/2. The list
of available parameters is stored in module params_db.

@author Miky Zamalloa
@license GPL

*/

:- module(costabs_params,[set_default_costabs_params/0, 
                          set_costabs_params/1, costabs_param/2,
			  cleanup_costabs_params/0]).


:- use_module(params_info_db,[param_info/3]).


:- dynamic param/2. % param(Name,Value) -> The curent value for Name is Value


set_default_costabs_params :-
        cleanup_costabs_params,
	findall(Name = Default,param_info(Name,_,Default),Params),
        set_costabs_params(Params).

set_costabs_params([]) :- !.
set_costabs_params([Name = Value|Ps]) :- !,
        set_costabs_param(Name,Value),
        set_costabs_params(Ps).
set_costabs_params(Params) :-
	throw(wrong_costabs_parameter(Params)).


costabs_param(Name,Value) :-
	param(Name,_), !,
        param(Name,Value).
costabs_param(Name,_Value) :-
	throw(wrong_costabs_parameter(Name)).


set_costabs_param(Name,Value) :-
	ground(Name),
	ground(Value),
	param_info(Name,Values,_),
	member(Value,Values),!,
        retractall(param(Name,_)),
        assertz(param(Name,Value)).
set_costabs_param(Name,Value) :-
	throw(costabs_exception(wrong_costabs_parameter(Name,Value))).


cleanup_costabs_params :-
	retractall(param(_,_)).
