



:- module(field_assertions_analysis,[field_assertions_analysis/0]).
:- use_module(field_assertions,[requires/2, ensures/3, all_params/1]).
%TODO uncomment the dynamic definition of require and ensure in the other file
:- use_module(rbr_abs_db,[abs_rule/6, 
			  call_rel/2, 
			  rule_field/2,
			  field_type/3]).
:- use_module(loaded_abs_db,[class/3]).
:- use_module(abs2rbr_compiler,[build_method_rule_name/3]).
:- use_module(stdlib(utils),[ut_list_union/3,ut_difference/3,ut_atom_codes/2,ut_member/2,ut_append/3]).

:-dynamic untouched_fields/2.%untouched_fields(class_name,list_of_fields)
:-dynamic touched_fields/2.%touched_fields(class_name,list_of_fields)
:-dynamic init_rule/1.%rules that belong to a constructor


field_assertions_analysis:-
	assertz(all_params([a(x),a(y)])),
	compute_untouched_fields,
	%print_untouched_fields,
	compute_requires,
	compute_ensures.
	%print_requires,
	%print_ensures.
	
compute_requires:-
	abs_rule(Name,_This,_InArgs,_OutArgs,_Body,[method,Class,_]),
	class(Class,_,AllFields_decl),
	extract_fields_from_decl(AllFields_decl,AllFields),
	get_field_bounds(AllFields,Class,Bounds),
	assertz(requires(Name,Bounds)),
	fail.
compute_requires.

compute_ensures:-
	abs_rule(_Name,_This,_InArgs,_OutArgs,Body,[_Kind,Class,_]),
	Class \==none,%the rule belongs to a class
	compute_rule_ensures(Body,Class),
	fail.
compute_ensures.



compute_rule_ensures([],_Class).
compute_rule_ensures([await(Id,_G)|Stmts],Class):-!,
	untouched_fields(Class,Untouched_fields),
	class(Class,_,AllFields_decl),
	extract_fields_from_decl(AllFields_decl,AllFields),
	get_field_bounds(AllFields,Class,Bounds),
	assertz(ensures(Id,Untouched_fields,Bounds)),
	compute_rule_ensures(Stmts,Class).
compute_rule_ensures([release(Id)|Stmts],Class):-!,
	untouched_fields(Class,Untouched_fields),
	class(Class,_,AllFields_decl),
	extract_fields_from_decl(AllFields_decl,AllFields),
	get_field_bounds(AllFields,Class,Bounds),
	assertz(ensures(Id,Untouched_fields,Bounds)),
	compute_rule_ensures(Stmts,Class).
compute_rule_ensures([_|Stmts],Class):-
	compute_rule_ensures(Stmts,Class).
	
	
get_field_bounds(AllFields,Class,Bounds):-
	get_field_bounds_(AllFields,Class,[],Bounds).
get_field_bounds_([],_Class,Accum,Accum).
get_field_bounds_([Field|Fields],Class,Accum,Bounds):-
	Accum_out=[a(x) >= Field,Field >= a(y)|Accum],
	get_field_bounds_(Fields,Class,Accum_out,Bounds).

compute_untouched_fields:-
	field_assertions_cleanup,
	compute_init_rules,%TODO we may need to remove this
	compute_touched_fields,
	compute_difference_fields.

field_assertions_cleanup:-
	retractall(init_rule(_)),
	retractall(untouched_fields(_,_)),
	retractall(touched_fields(_,_)).

compute_touched_fields:-
	abs_rule(Name,_This,_InArgs,_OutArgs,Body,[_Kind,Class,_]),
	Class \==none,%the rule belongs to a class
	\+init_rule(Name),%TODO we may need to remove this
	compute_rule_touched_fields(Body,Touched_fields),
	join_touched_fields(Class,Touched_fields),
	fail.
compute_touched_fields.

compute_rule_touched_fields(Body,Touched_fields):-
	compute_rule_touched_fields_(Body,[],Touched_fields).

compute_rule_touched_fields_([],Accum,Accum).
compute_rule_touched_fields_([field(F)=_B|Stmts],Accum,Touched_fields):-!,
	compute_rule_touched_fields_(Stmts,[field(F)|Accum],Touched_fields).
compute_rule_touched_fields_([_|Stmts],Accum,Touched_fields):-
	compute_rule_touched_fields_(Stmts,Accum,Touched_fields).

join_touched_fields(Class,Touched_fields2):-
	touched_fields(Class,Touched_fields1),!,
	ut_list_union(Touched_fields1,Touched_fields2,Touched_fields3),
	retract(touched_fields(Class,Touched_fields1)),
	assertz(touched_fields(Class,Touched_fields3)).

join_touched_fields(Class,Touched_fields):-
	assertz(touched_fields(Class,Touched_fields)).

compute_difference_fields:-
	touched_fields(Class_name,Touched_fields),
	class(Class_name,_,AllFields_declaration),
	extract_fields_from_decl(AllFields_declaration,AllFields),
	ut_difference(AllFields,Touched_fields,Untouched_fields),
	assertz(untouched_fields(Class_name,Untouched_fields)),
	fail.
compute_difference_fields.

compute_init_rules:-
	class(Class_name,_,_AllFields),
	build_method_rule_name(methodSig(init,_RetType,_Params),Class_name,RuleName),
	compute_init_rule(RuleName),
	fail.
compute_init_rules.

compute_init_rule(RuleName):-
	init_rule(RuleName),!.
compute_init_rule(RuleName):-
	abs_rule(RuleName,_This,_InArgs,_OutArgs,Body,_Info),
	assertz(init_rule(RuleName)),
	compute_init_rules_body(Body).
	
compute_init_rules_body([]).
compute_init_rules_body([call(Kind,RuleCall)|Stmts]):-
	ut_member(Kind,[case,if,cont,entrywhile,while,awguard]),!,
	RuleCall=..[RuleName|_],
	compute_init_rule(RuleName),
	compute_init_rules_body(Stmts).
compute_init_rules_body([_|Stmts]):-
	compute_init_rules_body(Stmts).


extract_fields_from_decl([],[]).
extract_fields_from_decl([field(Name,_Type,_Init)|Field_decls],[field(Name)|Fields]):-
	extract_fields_from_decl(Field_decls,Fields).
%taken from abs_program_loader
is_method_entry(Entry,MethodName,ClassName):-
	ut_atom_codes(Entry,EntryCodes),
	ut_split_path_by_code(EntryCodes,46,[MethodName,ClassName]).


print_untouched_fields:-
	untouched_fields(Class_name,Fields),
	writeln(untouched_fields(Class_name,Fields)),
	fail.
print_untouched_fields.


print_requires:-
	requires(Class,Bounds),
	writeln(requires(Class,Bounds)),
	fail.
print_requires.

print_ensures:-
	ensures(Id,UntouchedFields,Bounds),
	writeln(ensures(Id,UntouchedFields,Bounds)),
	fail.
print_ensures.

%generalization of ut_split_path
%Should it be included in utils?
ut_split_path_by_code([],_Code,[]).
ut_split_path_by_code(CP,Code,CPL):-
	ut_append(Pre,[Code|Post],CP),!, 
	atom_codes(Dir,Pre),
	CPL = [Dir|More],
	ut_split_path_by_code(Post,Code,More).
ut_split_path_by_code(CP,_Code,[CPA]):-
	atom_codes(CPA,CP).
	
