package costabs.handlers;

import java.io.File;
import java.util.ArrayList;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.handlers.HandlerUtil;

import costabs.console.ConsoleHandler;
import costabs.console.CostabsShellCommand;
import costabs.dialogs.MethodDialog;
import costabs.utils.CostabsParser;
import costabs.utils.SourceUtils;

/**
 * Our sample handler extends AbstractHandler, an IHandler base class.
 * @see org.eclipse.core.commands.IHandler
 * @see org.eclipse.core.commands.AbstractHandler
 */
public class costabsHandler extends AbstractHandler {

	private static final String ABS_PL_PATH = "/tmp/costabs/absPL/abs.pl";

	/**
	 * The constructor.
	 */
	public costabsHandler() {
	}

	/**
	 * the command has been executed, so extract the needed information
	 * from the application context.
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {

		final Shell shell= HandlerUtil.getActiveWorkbenchWindowChecked(event).getShell();
		
		// Generate abs.pl to parse it to get methods and functions
		generate();
		CostabsParser parser = new CostabsParser(ABS_PL_PATH);
		
		// Show the dialog and wait for the user answer
		MethodDialog mDialog = new MethodDialog (shell, parser.getMethods(), parser.getFunctions(), false);
		mDialog.open();

		if (mDialog.getReturnCode() == MethodDialog.CANCEL) {
			ConsoleHandler.write("Don't do anything, cancelled by the user");
		}else{
			// If analyze, get preferences and run
			analyze(mDialog.getSelectedMethods());
		}
		return null;
	}

	private void generate() {		

		ConsoleHandler.clearConsole();
		
		CostabsShellCommand shell = new CostabsShellCommand();

		String absFile = SourceUtils.extractResource(SourceUtils.obtainActiveEditor()).getLocation().toString();

		doCostabsDirectory();

		shell.generateProlog(absFile,false);

		ConsoleHandler.write(shell.getResult());
		//ConsoleHandler.write(shell.getError());

	}

	private void analyze(ArrayList<String> methods) {		

		ConsoleHandler.clearConsole();
		
		CostabsShellCommand command = new CostabsShellCommand();

		String absFile = SourceUtils.extractResource(SourceUtils.obtainActiveEditor()).getLocation().toString();

		doCostabsDirectory();

		command.generateProlog(absFile, false);
		command.analyze(absFile, methods);

		ConsoleHandler.write(command.getResult());
		//ConsoleHandler.write(command.getError());
		
		
	}
	
	private void doCostabsDirectory() {
		File f = new File("//tmp//costabs//absPL");
		f.mkdirs();
	}


}
