
:- module(costabs_command_line,[costabs_shell_main/1,main/1]).

:- use_module(costabs(costabs_driver),[costabs/3]).
:- use_module(stdlib(utils),[ut_atom_codes/2,ut_file_exists/1,ut_member/2,ut_atoms_concat/2,ut_split_path/2]).
:- use_module(stdlib(shell_utilities),[get_env_atom/2,make_dirpath/1]).
:- use_module(costabs(costabs_settings),[costabs_path/1, abspath/1,abs_frontend/1]).



main(Args) :-
	costabs_shell_main(Args).

costabs_shell_main([]) :- !,
	welcome_message.
costabs_shell_main(['-L']) :- !,
 	copying.
costabs_shell_main(['--help']) :- !,
 	usage_message.
costabs_shell_main(['-H']) :- !,
 	usage_message.
costabs_shell_main(['-h']) :- !,
 	usage_message.

costabs_shell_main(Args) :-
	expand_files_paths(Args,Args_1),
	parse_files(Args_1,ABSFiles,RestArgs1),
	parse_mode(RestArgs1,Mode,RestArgs2),
	costabs_mode(Mode,ABSFiles,RestArgs2).

costabs_mode(compile,ABSFiles,Args) :-
	call_PrologBackend(ABSFiles,Args).
costabs_mode(full,ABSFiles,Args) :-
	parse_entries(Args,Entries,RestArgs1),
	parse_classpath(RestArgs1,RestArgs2),
	parse_options(RestArgs2,Options),
	costabs(ABSFiles,Entries,Options).
costabs_mode(analyze,[],Args) :-
	parse_entries(Args,Entries,RestArgs),
	parse_options(RestArgs,Options), 
	costabs([],Entries,Options).


%%%%%%%%%%%%%%%%%%%%%%% Parsing %%%%%%%%%%%%%%%%%%%%%%%

parse_files(Args,Files,RestArgs) :- parse_names(Args,Files,RestArgs).

parse_entries(['-entries'|Args],Entries,MoreArgs) :- !,
	parse_names(Args,Entries,MoreArgs).
parse_entries(Args,[],Args).

parse_names([],[],[]) :- !. 
parse_names([Name|Args],[Name|MoreNames],MoreArgs) :-
	is_not_flag([Name|Args]), !,
	parse_names(Args,MoreNames,MoreArgs).
parse_names(Args,[],Args).


parse_mode(['-mode',Mode|MoreArgs],Mode,MoreArgs) :- !.
parse_mode(Args,'full',Args).


% This does not do anything! Check and delete
parse_classpath(['-cp',_K|MoreArgs],MoreArgs) :- !.
parse_classpath(Args,Args).


parse_options([],[]).
parse_options([Opt,K|MoreArgs],[cost_model = K|MoreOptions]) :-
	(Opt == '-cm' ; Opt == '-cost_model'),!,
	parse_options(MoreArgs,MoreOptions).
parse_options([Opt,K|MoreArgs],[cost_centers = K|MoreOptions]) :-
	(Opt == '-cc' ; Opt == '-cost_centers'),!,
	parse_options(MoreArgs,MoreOptions).
parse_options([Opt,K|MoreArgs],[size_abst= K|MoreOptions]) :-
	Opt == '-norm', !,
	parse_options(MoreArgs,MoreOptions).
parse_options([Opt,K|MoreArgs],[stdlib = K|MoreOptions]) :-
	Opt == '-stdlib', !,
	parse_options(MoreArgs,MoreOptions).
parse_options([Opt,K_at|MoreArgs],[verbosity = K|MoreOptions]) :- 
	(Opt == '-v' ; Opt == '-verbosity'),!,
	atom_number(K_at,K),
	parse_options(MoreArgs,MoreOptions).
parse_options([Opt|MoreArgs],[debug = yes|MoreOptions]) :- 
	(Opt == '-debug'; Opt == '-d'), !,
	parse_options(MoreArgs,MoreOptions).
parse_options([Opt|MoreArgs],[asymp = yes|MoreOptions]) :- 
	(Opt == '-aymp'; Opt == '-a'), !,
	parse_options(MoreArgs,MoreOptions).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%% Calling the Prolog backend %%%%%%%%%%%%%

call_PrologBackend(ABSFiles,Args) :-
	parse_prologBackend_opts(Args,[CP,StdLib]),
	(var(CP) -> abs_frontend(CP) ; true), % Loading default value for CP
	(var(StdLib) -> StdLib = ' ' ; true), % Loading default value for StdLib
	ABSFiles = [ABSFile], % By now we assume there can only be one ABS source file
	ut_atoms_concat(['java -cp ',CP,' abs.backend.prolog.PrologBackend -d /tmp/costabs/absPL',StdLib,ABSFile],Command),
	make_dirpath('/tmp/costabs/absPL'),
	shell(Command),
	generateTxtFiles.

parse_prologBackend_opts([],[_,_]).
parse_prologBackend_opts(['-cp',CP|MoreArgs],[CP,StdLib]) :- !,
	parse_prologBackend_opts(MoreArgs,[_,StdLib]).
parse_prologBackend_opts(['-stdlib',YesOrNo|MoreArgs],[CP,StdLib]) :- !,
	(YesOrNo = yes -> StdLib = ' ' ; StdLib = '-nostdlib'),
	parse_prologBackend_opts(MoreArgs,[CP,_]).


%%%%%% Generation of txt files with functions and methods %%%%%%

%:- dynamic mainBlock/2.
%:- dynamic def/4.
%:- dynamic methodImpl/4.

generateTxtFiles :-
%	initGenerate,
	['/tmp/costabs/absPL/abs.pl'], % bySamir: I replaced this to avoid warning when loading abs.pl
	% byMiky: I put it back since it was not working and the warnings are actually never shown
	%load_abs_prolog_file('/tmp/costabs/absPL/abs.pl'),	
	open('/tmp/costabs/absPL/absFunctions.txt',write,SOutFunctions),	
	functions_to_txt(SOutFunctions),
	close(SOutFunctions),
	open('/tmp/costabs/absPL/absMethods.txt',write,SOutMethods),
	mainBlock_to_txt(SOutMethods),	
	methods_to_txt(SOutMethods),
	close(SOutMethods).

initGenerate :-
	assert(mainBlock(0,_)),
	assert(def(0,_,_,_)),
	assert(methodImpl(0,_,_,_)).

functions_to_txt(_) :-
	def(0,_,_,_), !.
functions_to_txt(SOut) :-
	def(Function,_,_,_),
	format(SOut,"~p~n",[Function]),
	fail.
functions_to_txt(_).	

mainBlock_to_txt(_) :-	
	mainBlock(0,_), !.
mainBlock_to_txt(SOut) :-	
	mainBlock(_,_),
	format(SOut,"main~n",[]).

methods_to_txt(_) :-
	methodImpl(0,_,_,_), !.
methods_to_txt(SOut) :-
	methodImpl(methodSig(Method,_,_),Class,_,_),
	format(SOut,"~p.~p~n",[Class,Method]),
	fail.
methods_to_txt(_).


%%%%%%%%%%%%%%%%% Auxiliary predicates %%%%%%%%%%%%%%%%%

is_flag([Flag|_]) :- atom_concat('-',_,Flag).

is_not_flag(Args) :- \+ is_flag(Args).


welcome_message :-
	format("~nThis is COSTABS, version 1.0.~n",[]),
	format("Copyright 2011, 2011 E.Albert, P.Arenas, S.Genaim, M.Gomez-Zamalloa and G.Puebla.~n",[]),
	format("This program comes with ABSOLUTELY NO WARRANTY;for details execute 'costabs -W'~n",[]),
	format("This is free software, and you are welcome to redistribute it undercertain conditions;~n",[]),
	format("For details execute 'costabs -L'~n",[]),
	format("For usage information execute 'costabs -h'~n",[]).

usage_message :-
	format("~nUsage: costabs ABSFile -entries Entries [Options]~n",[]),
	format("    where:~n",[]),
	format("        ABSFile is the ABS source file name. Either set accordingly the ~n",[]),
	format("            ABSPath environment variable or provide and absolute path~n",[]),
	format("        Entries is the list of methods/functions to be analyzed~n",[]),
	format("            each using the syntax: FunctionName or ClassName.MethodName~n",[]),
	format("        Options are of the form: [-cm CostModel] [-norm SizeNorm] [-d]~n",[]),
        format("                                 [-cc CostCenters] [-a] [-v VerbLevel]~n~n",[]),
	format("Meaning of options (the first value is the default one):~n",[]),
	format("    -cm CostModel: Sets the cost-model (note that termination is considered a cost-model)~n",[]),
	format("                CostModel is in {steps,termination,memory,objects}~n",[]),
	format("    -cc CostCenters: Analysis per cost centers or not~n",[]),
	format("                CostCenters is in {no,yes}~n",[]),
	format("    -norm SizeNorm: Sets the size norm~n",[]),
	format("                SizeNorm is in {term_size,term_depth}~n",[]),
	format("    -d: Enables debugging mode (by default it is disabled)~n",[]),
	format("    -a: Obtains asymptotic upper-bounds (by default it is disabled)~n",[]),
	format("    -v VerbLevel: Specifies the verbosity level of the output~n",[]),
	format("                VerbLevel is in {2,0,1}~n~n",[]),
	format("Example: costabs PeerToPeer.abs -entries Node.reqFile -cm memory -cc yes~n",[]).
%	format("   Analyze method reqFile of class Node with cost-model memory with cost-centers~n",[]).

copying :-
	File = '~/Systems/costa/COPYING',
 	atom_concat('cat ',File,Command),
	shell(Command).



%%
%% bySamir: this expands the abs files with full path, this make some
%%          code in abs_program_loader redundant, to be removed after
%%          asking Miky
%%

expand_files_paths([],[]).
expand_files_paths([F|As],[EF|EAs]) :-
	is_not_flag([F|As]),
	!,
	expand_file_path(F,EF),
	expand_files_paths(As,EAs).
expand_files_paths(As,As).


expand_file_path(F,EF) :-
	ut_file_exists(F),
	!,
	F=EF.
expand_file_path(F,EF) :-
	costabs_path(P),
	ut_atoms_concat([P,'examples/',F],EF),
	ut_file_exists(EF),
	!.
expand_file_path(F,EF) :-
	abspath(P),
	ut_atoms_concat([P,'/',F],EF),
	ut_file_exists(EF),
	!.
expand_file_path(F,_) :-
	throw(costabs_err(file_not_found(F))).


load_abs_prolog_file(F) :-
	open(F,read,S),
	load_abs_prolog_file_aux(S),
	close(S).

load_abs_prolog_file_aux(S) :-
	read(S,T),
	(T == end_of_file ->
	    true
	;
	    assertz(T),
	    load_abs_prolog_file_aux(S)
	).
