<?php
class CostabsWeb {
		
	// Folder in which to store tmp files
	const TMP_PATH = "/tmp/costabs/";
	// Folder to store abs.pl from generateProlog
	const ABS_PL_FOLDER = "/tmp/costabs/absPL/";
	// Path to examples folder and the file with the examples to show in web
	const EXAMPLES_ABS_DIR = "../../../../examples/";
	const EXAMPLES_FILE = "showWeb.txt";
	// Path to the user setting
	const INSTALL_SETTINGS = "../../install_settings";

	/**
	 * Generate the html tags for examples. 
	 */
	public function printExamples() {
		echo "<ul>";
	
		$file_examples = fopen(self::EXAMPLES_ABS_DIR.self::EXAMPLES_FILE,"r");

		while ($line_example = fgets($file_examples)) {
			list($example_name,$stdlib_option) = explode("=",$line_example,2);
			if (($example_name != "") && ($stdlib_option != ""))
				$element = "<li><a class=\"links\" href=\"analyzer.php?file=";
				$element = $element.$example_name."&stdlibExample=".$stdlib_option."\">$example_name</a>";
				echo $element;
		}
		fclose($file_examples);

		echo "</ul>";
	}

	/**
	 * Generate the html tags for the option to choose the use of standard library.
	 * If there was an example choosen, it is set to the value that needs to execute. 
	 */
	public function printStdlibOption($exampleLibrary) {

		echo '<b>a) </b>Use standard library:</br></br>';
		echo '<input type="radio" name="stdlib" value="yes"'; 

		if ($exampleLibrary == "-nostdlib") {
			echo '/>yes';
			echo '<input type="radio" name="stdlib" value="no" checked/>no</br></br>';
		}
		else {
			echo 'checked/>yes';
			echo '<input type="radio" name="stdlib" value="no"/>no</br></br>';
		}
	}

	/**
	 * Remove the extension from a file name. 
	 */
	public function remove_extension($strName) {
		$ext = strrchr($strName, '.');
    	if($ext !== false)
				$strName = substr($strName, 0, -strlen($ext));
		return $strName;
	}

	/**
	 * Store data to a file. 
	 */
	public function file_put_contents($filename, $data, $respect_lock = true) {
		// Open the file for writing
		$fh = @fopen($filename, 'w');
		if ($fh === false)
 			return false;

		// Check to see if we want to make sure the file is locked before we write to it
		if ($respect_lock === true && !flock($fh, LOCK_EX)) {
			fclose($fh);
			return false;
		}

		// Convert the data to an acceptable string format
		if (is_array($data))
			$data = implode('', $data);
		else
			$data = (string) $data;

		// Write the data to the file and close it
		$bytes = fwrite($fh, $data);

		// This will implicitly unlock the file if it's locked
		fclose($fh);

		return $bytes;
	}

	/**
	 * Generate a name for a tmp file. 
	 */
	public function tempnam_sfx($path, $suffix) {
		do {
			$file = $path.mt_rand().$suffix;
			$fp = @fopen($file, 'x');
		}
		while(!$fp);
		fclose($fp);
		return $file;
	} 

	/**
	 * Get the functions's names from a file. 
	 */
	public function loadFunctions() {
		return $this->loadNames("absFunctions.txt");
	}

	/**
	 * Get the methods's names from a file. 
	 */
	public function loadMethods() {
		return $this->loadNames("absMethods.txt");
	}

	/**
	 * Generate a list of names from a file. 
	 */
	public function loadNames($file) {
		$result = array();
		$i = 0;
		$file = fopen(self::ABS_PL_FOLDER.$file,"r");

		while ($line = fgets($file)) {
			if (sizeof($line) > 0) { $result[$i] = trim($line); $i++; } 		
		}
		fclose($file);

		return $result;
	}

	/**
	 * Get the user's home path.
	 */
	public function getHome() {
		$file = fopen(self::INSTALL_SETTINGS,"r");
		$line = fgets($file);	
		list($setting, $result) = split('=', $line);	 		
		fclose($file);
		$result = trim($result);
		return $result;
	}

}

$config = new CostabsWeb();

?>
