/** <module> The Costabs UBs database

This module stores the UBs obtained by Costabs as a set of facts of the form

  costabs_ub(Entry,CC,UB), meaning:

UB is the ub associated with cost center CC for entry Entry, obtained
by Costabs
 
@author Miky Zamalloa
@license GPL

*/

:- module(ubs_db,[store_ub/3,costabs_ub/3,cleanup_ubs/0]).
:- use_module(pubs(asymptotic_reduction),[asymp_reduction/3]).
:- use_module(costabs_params,[costabs_param/2]).

:- dynamic costabs_ub/3.


store_ub(E,CC,UB) :-
	(costabs_param(asymp,no) -> AUB=UB ; asymp_reduction(UB,[],AUB)),
	assertz(costabs_ub(E,CC,AUB)).

cleanup_ubs :- retractall(costabs_ub(_,_,_)).


